unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, VCLTee.TeEngine, VCLTee.TeeTools,
  VCLTee.Series, Vcl.ExtCtrls, VCLTee.TeeProcs, VCLTee.Chart;

type
  TForm1 = class(TForm)
    chrtTSPAllocation: TChart;
    Series2: TPieSeries;
    Series1: TPieSeries;
    procedure Series1BeforeDrawValues(Sender: TObject);
    function GetCurrentContribRect(aChart: TChart): TRect;
    function GetChartHMargin(aChart: TChart): integer;
    function GetChartVMargin(aChart: TChart): integer;
    procedure Series2BeforeDrawValues(Sender: TObject);

  private
    procedure ConfigChart(aChart: TChart);
    procedure DrawPieSeriesCaption(aChart: TChart; const aCaption: String);
    { Private declarations }
  public
    { Public declarations }
    constructor Create(aOwner: TComponent); override;
  end;

var
  Form1: TForm1;

implementation

uses
  math, VCLTee.TeCanvas;

{$R *.dfm}

constructor TForm1.Create(aOwner: TComponent);
begin
  inherited;

  ConfigChart(chrtTSPAllocation);

  Series2.FillSampleValues(5);
  Series1.FillSampleValues(5);


end;

procedure TForm1.DrawPieSeriesCaption(aChart: TChart; const aCaption: String);
var
  lC: TCanvas3D;
  lR: TRect;
  lTop: Integer;
  lLeft: integer;
begin
  lR := aChart.ChartRect;


  lC := aChart.Canvas;
  lC.Font.Assign(aChart.Title.Font);

  lTop := (lR.Top - lC.TextHeight('Ig')) div 2;

  lLeft := lR.Left + ((lR.Right-lR.Left - lC.TextWidth(aCaption)) div 2);
  lC.TextOut(lLeft, lTop, aCaption);
end;


function TForm1.GetChartHMargin(aChart: TChart): integer;
begin
  result := Min(aChart.Width div 20,aChart.Width div 20);
end;

function TForm1.GetChartVMargin(aChart: TChart): integer;
begin
  result := Min(aChart.Width div 10,aChart.Width div 10);
end;

function TForm1.GetCurrentContribRect(aChart: TChart): TRect;
var
  lHMargin: integer;
  lCaptionHeight: Integer;
  lVMargin: Integer;
begin

    lHMargin := GetChartHMargin(aChart);
    lVMargin := GetChartVMargin(aChart);
    lCaptionHeight := aChart.Canvas.TextHeight('T');
    result := Rect(lHMargin div 2, lCaptionHeight + lHMargin, (aChart.Width - lHMargin) div 2, aChart.Legend.Top - lVMargin);
end;

procedure TForm1.ConfigChart(aChart: TChart);
var
 lAxis: TChartAxis;
begin
  aChart.Title.Font.Size := 12;
  aChart.Title.Font.Style := [fsBold];
  aChart.Title.Font.Color := clNavy;

  lAxis := aChart.LeftAxis;

  lAxis.Title.Font.Size := 10;
  lAxis.Title.Font.Style := [fsBold];

  lAxis.LabelsFont.Size := 9;
  lAxis.LabelsSeparation := 100;
  lAxis.Axis.Width := 1;
  lAxis.Ticks.Visible := False;
  lAxis.MinorTicks.Visible := False;
  lAxis.TicksInner.Visible := False;
  lAxis.Grid.Color := clSilver;
  lAxis.Grid.Style := psDot;

  lAxis := aChart.BottomAxis;

  lAxis.Title.Font.Size := 10;
  lAxis.Title.Font.Style := [fsBold];
  lAxis.Axis.Width := 1;
  lAxis.Grid.Visible := False;
  lAxis.MinorTicks.Visible := False;
  lAxis.TicksInner.Visible := False;

  lAxis.LabelsFont.Size := 9;
  lAxis.LabelsSeparation := 50;

  aChart.Color := clWhite;
  aChart.RightAxis.Visible := False;
  aChart.TopAxis.Visible := False;
  aChart.BackWall.Visible := False;

  aChart.LeftWall.Color := clWhite;

  aChart.Legend.Font.Size := 9;
  aChart.Legend.Symbol.Squared := True;
  aChart.Legend.Symbol.Pen.Visible := False;
  aChart.Legend.Gradient.Visible := False;


  aChart.View3D := False;
  aChart.Gradient.Visible := False;
end;

procedure TForm1.Series1BeforeDrawValues(Sender: TObject);
var
  lSrs: TPieSeries;
  lChart: TChart;
begin
  Assert(Sender is TPieSeries);
  lSrs := TPieSeries(Sender);
  Assert(lSrs.ParentChart is TChart);


  lChart := TChart(lSrs.ParentChart);
  lChart.ChartRect := GetCurrentContribRect(lChart);

    DrawPieSeriesCaption(lChart, lSrs.Title);
end;

procedure TForm1.Series2BeforeDrawValues(Sender: TObject);
var
  lSrs: TPieSeries;
  lChart: TChart;
  lHMargin: Integer;
  lVMargin: Integer;
  lCaptionHeight: Integer;
begin
  Assert(Sender is TPieSeries);
  lSrs := TPieSeries(Sender);
  Assert(lSrs.ParentChart is TChart);
  lChart := TChart(lSrs.ParentChart);

  lVMargin := GetChartVMargin(lChart);;
  lHMargin := GetChartHMargin(lChart);

  lChart.Canvas.Font.Assign(lChart.Title.Font);
  lCaptionHeight := lChart.Canvas.TextHeight('T');
  lChart.ChartRect :=
    Rect((lChart.Width - lHMargin) div 2,
         lCaptionHeight + lHMargin,
         lChart.Width - (lHMargin div 2),
         lChart.Legend.Top - lVMargin);

  DrawPieSeriesCaption(lChart, lSrs.Title);
end;

end.
